@@ -1,5 +1,5 @@
 /**********************************************************************
-Copyright (C) 2003-2005 Andreas Rottmann
+Copyright (C) 2003-2005, 2009, 2010 Andreas Rottmann
  
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as
@@ -36,7 +36,8 @@ USA.
 
 static SCM is_a_p_proc = SCM_UNSPECIFIED;
 static SCM module_add_x = SCM_UNSPECIFIED;
-static SCM scm_sym_make = SCM_UNSPECIFIED;
+static SCM var_make = SCM_UNSPECIFIED;
+static SCM var_add_method_x = SCM_UNSPECIFIED;
 
 /* TODO: Use snarfer for kewords & symbols */
 static SCM k_specializers = SCM_UNSPECIFIED;
@@ -197,6 +198,9 @@ gw_guile_add_subr_method (SCM generic, SCM subr, SCM all_specializers,
     formals = scm_cons (scm_from_locale_symbol (buffer), formals);
   }
 
+#if SCM_MAJOR_VERSION <= 1 && SCM_MINOR_VERSION < 9
+  /* in Guile 1.8 and before, the procedure for a method had to be an
+     interpreted closure. */
   if (use_optional_args)
   {
     SCM f_apply = scm_c_eval_string ("apply");
@@ -213,12 +217,15 @@ gw_guile_add_subr_method (SCM generic, SCM subr, SCM all_specializers,
     procm = scm_closure (scm_list_2 (formals, scm_cons (subr, formals)),
                          scm_top_level_env (SCM_TOP_LEVEL_LOOKUP_CLOSURE));
   }
+#else
+  procm = subr;
+#endif
 
-  meth = scm_apply_0 (scm_sym_make,
+  meth = scm_apply_0 (SCM_VARIABLE_REF (var_make),
                       scm_list_5 (scm_class_method,
                                   k_specializers, specializers,
                                   k_procedure, procm));
-  scm_add_method (generic, meth);
+  scm_call_2 (SCM_VARIABLE_REF (var_add_method_x), generic, meth);
 } 
 
 /* What's going on here?
@@ -343,7 +350,7 @@ allocate_generic_variable (SCM module, SCM sym)
 
   if (scm_is_false (var)) {
     /* Symbol unbound, make a new generic */
-    generic = scm_apply_0 (scm_sym_make,
+    generic = scm_apply_0 (SCM_VARIABLE_REF (var_make),
                            scm_list_3 (scm_class_generic, k_name, sym));
     return scm_make_variable (generic);
   } else if (scm_is_true (scm_call_2 (is_a_p_proc, scm_variable_ref (var),
@@ -355,7 +362,7 @@ allocate_generic_variable (SCM module, SCM sym)
   } else if (scm_is_true (scm_procedure_p (scm_variable_ref (var)))) {
     /* Make a generic that falls back on the original binding. NB: generics also
        satisfy procedure?. */
-    generic = scm_apply_0 (scm_sym_make,
+    generic = scm_apply_0 (SCM_VARIABLE_REF (var_make),
                            scm_list_5 (scm_class_generic,
                                        k_name, sym,
                                        k_default, scm_variable_ref (var)));
@@ -456,7 +463,7 @@ gw_guile_ensure_latent_variables_hash_and_binder (SCM module)
       return SCM_BOOL_F; /* won't get here */
     }
 
-    scm_struct_set_x (module, SCM_MAKINUM (scm_module_index_binder),
+    scm_struct_set_x (module, scm_from_int (scm_module_index_binder),
                       scm_c_make_gsubr ("%gw-module-binder", 3, 0,
                                         0, gw_module_binder_proc));
 
@@ -531,11 +538,12 @@ gw_guile_procedure_to_method_public (SCM proc, SCM specializers,
   SCM pair;
   SCM existing_latents;
   SCM entry;
+  int c_n_req_args;
 
   SCM_VALIDATE_PROC (1, proc);
   SCM_VALIDATE_LIST (2, specializers);
   SCM_VALIDATE_SYMBOL (3, generic_name);
-  SCM_VALIDATE_INUM (4, n_req_args);
+  SCM_VALIDATE_INT_COPY (4, n_req_args, c_n_req_args);
   /* the fifth is a bool */
   
   generics = gw_guile_ensure_generics_module ();
@@ -553,7 +561,7 @@ gw_guile_procedure_to_method_public (SCM proc, SCM specializers,
                                 proc,
                                 specializers,
                                 scm_current_module (),
-                                scm_to_int (n_req_args), 
+                                c_n_req_args,
                                 scm_is_true (use_optional_args));
       return;
     }
@@ -729,11 +737,11 @@ gw_guile_handle_wrapper_error(GWLangArena arena,
   static SCM wrong_type_key = SCM_BOOL_F;
 
   if (scm_is_false (out_of_range_key))
-    out_of_range_key = scm_permanent_object (
-            scm_c_make_keyword("out-of-range"));
+    out_of_range_key =
+      scm_permanent_object (scm_from_locale_keyword ("out-of-range"));
   if (scm_is_false (wrong_type_key))
-    wrong_type_key = scm_permanent_object (
-            scm_c_make_keyword("wrong-type"));
+    wrong_type_key =
+      scm_permanent_object (scm_from_locale_keyword ("wrong-type"));
   
   switch (error->status)
   {
@@ -872,20 +880,23 @@ gw_guile_runtime_init (void)
   {
     scm_load_goops();
 
-    scm_sym_make = scm_permanent_object (
-            SCM_VARIABLE_REF (scm_c_module_lookup (scm_module_goops,
-                                                   "make")));
+    var_make =
+      scm_permanent_object (scm_c_module_lookup (scm_module_goops, "make"));
+    var_add_method_x =
+      scm_permanent_object (scm_c_module_lookup (scm_module_goops, "add-method!"));
     is_a_p_proc = scm_permanent_object (
             SCM_VARIABLE_REF (scm_c_module_lookup (scm_module_goops,
                                                    "is-a?")));
     module_add_x = scm_permanent_object (
             SCM_VARIABLE_REF (scm_c_lookup ("module-add!")));
-    k_specializers = scm_permanent_object (
-            scm_c_make_keyword ("specializers"));
-    k_procedure = scm_permanent_object(
-            scm_c_make_keyword ("procedure"));
-    k_name = scm_permanent_object( scm_c_make_keyword ("name"));
-    k_default = scm_permanent_object (scm_c_make_keyword ("default"));
+    k_specializers = 
+      scm_permanent_object (scm_from_locale_keyword ("specializers"));
+    k_procedure = 
+      scm_permanent_object (scm_from_locale_keyword ("procedure"));
+    k_name =
+      scm_permanent_object (scm_from_locale_keyword ("name"));
+    k_default =
+      scm_permanent_object (scm_from_locale_keyword ("default"));
     sym_generic = scm_permanent_object (scm_from_locale_symbol ("generic"));
     sym_class = scm_permanent_object (scm_from_locale_symbol ("class"));
     sym_sys_gw_latent_variables_hash = scm_permanent_object
